--
-- MinimapIcon
--
-- Copyright (c) 2021 Dana Basken
--

MinimapIcon = {}
MinimapIcon.__index = MinimapIcon

function MinimapIcon:Create(id, icon, db, onLeftClick)
  local instance = {}
  setmetatable(instance, MinimapIcon)
  instance:Construct(id, icon, db, onLeftClick)
  return instance
end

function MinimapIcon:Construct(id, icon, db, onLeftClick)

  self.id = id
  self.db = db
  self.onLeftClick = onLeftClick

  self.dataBroker = LibStub("LibDataBroker-1.1"):NewDataObject(self.id, {
    type = "data source",
    text = id,
    icon = icon,
    OnEnter = function() self:OnEnter() end,
    OnLeave = function() self:OnLeave() end,
    OnClick = function(_, button) self:OnClick(button) end
  })

  self.icon = LibStub("LibDBIcon-1.0")
  self.icon:Register(self.id, self.dataBroker, self.db.char.minimap)

end

function MinimapIcon:OnEnter()
  -- TODO: show info frame
end

function MinimapIcon:OnLeave()
  -- TODO: hide info frame
end

function MinimapIcon:OnClick(button)
  if button == "LeftButton" then
    self.onLeftClick()
  end
  if button == "RightButton" then
    -- TODO: menu
  end
end

function MinimapIcon:UpdateVisibility()
  if self.db.char.minimap.hide then
    self:Hide()
  else
    self:Show()
  end
end

function MinimapIcon:Show()
  self.icon:Show(self.id)
  self.db.char.minimap.hide = false
end

function MinimapIcon:Hide()
  self.icon:Hide(self.id)
  self.db.char.minimap.hide = true
end

function MinimapIcon:Toggle()
  if self.db.char.minimap.hide then
    self:Show()
  else
    self:Hide()
  end
end
