--
-- ProgressBar
--
-- Copyright (c) 2021 Dana Basken
--

ProgressBar = {}
ProgressBar.__index = ProgressBar

function ProgressBar:Create(parentFrame, value, max)
  local instance = {}
  setmetatable(instance, ProgressBar)
  instance:Construct(parentFrame, value, max)
  return instance
end

function ProgressBar:Construct(parentFrame, value, max)
  self.parentFrame = parentFrame
  self.value = value
  self.max = max
  self.text = ""
  self:CreateUI()
  self:UpdateUI()
end

function ProgressBar:CreateUI()
  self:CreateContainerUI()
  self:CreateBarUI()
  self:CreateProgressUI()
  self:CreateLabelUI()
end

function ProgressBar:CreateContainerUI()
  self.frame = CreateFrame("Frame", "ProgressBar", self.parentFrame, BackdropTemplateMixin and "BackdropTemplate")
  self.frame:SetWidth(self.parentFrame:GetWidth())
  self.frame:SetHeight(self.parentFrame:GetHeight())
  self.frame:SetPoint("CENTER", 0, 0)
  self.frame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32})
  self.frame:SetBackdropColor(0, 0, 0, 0)
end

function ProgressBar:CreateBarUI()
  self.bar = CreateFrame("Frame", "ProgressBarBar", self.frame, BackdropTemplateMixin and "BackdropTemplate")
  self.bar:SetWidth(self.frame:GetWidth())
  self.bar:SetHeight(15)
  self.bar:SetPoint("CENTER", 0, 0)
  self.bar:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32})
  self.bar:SetBackdropColor(0, 0, 0, 1)
end

function ProgressBar:CreateProgressUI()
  local background = CreateFrame("Frame", "ProgressBarProgress", self.bar, BackdropTemplateMixin and "BackdropTemplate")
  background:SetWidth(self.bar:GetWidth())
  background:SetHeight(self.bar:GetHeight())
  background:SetPoint("CENTER", 0, 0)
  background:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32, insets = {left = 1, right = 1, top = 1, bottom = 1}})
  background:SetBackdropColor(0, 0, 0, 1)
  background:SetFrameLevel(self.frame:GetFrameLevel() + 1)

  self.progress = CreateFrame("Frame", "ProgressBarProgress", self.bar, BackdropTemplateMixin and "BackdropTemplate")
  self.progress:SetWidth(5)
  self.progress:SetHeight(self.bar:GetHeight())
  self.progress:SetPoint("LEFT", 0, 0)
  self.progress:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", tile = true, tileSize = 32, insets = {left = 1, right = 1, top = 1, bottom = 1}})
  self.progress:SetBackdropColor(0, 1, 0, 0.5)
  self.progress:SetFrameLevel(self.frame:GetFrameLevel() + 1)
end

function ProgressBar:CreateLabelUI()
  self.label = self.bar:CreateFontString("FontString", "OVERLAY")
  self.label:SetFont("Fonts\\FRIZQT__.TTF", 10, "MONOCHROME")
  self.label:SetPoint("CENTER", self.bar, 0, 0)
  self.label:SetTextColor(1.0, 1.0, 1.0, 0.75)
end

function ProgressBar:UpdateUI()
  local percent = self.value / self.max
  local width = self.frame:GetWidth() * percent
  self.label:SetText(self.text)
  self.progress:SetWidth(width)
end

function ProgressBar:GetFrame()
  return self.frame
end

function ProgressBar:Set(text, value, max)
  self.text = text
  self.value = value
  self.max = max
  self:UpdateUI()
end

function ProgressBar:SetValue(value)
  self.value = value
  self:UpdateUI()
end

function ProgressBar:SetMax(max)
  self.max = max
  self:UpdateUI()
end

function ProgressBar:SetText(text)
  self.text = text
  self:UpdateUI()
end
