--
-- StateMachine
--
-- Copyright (c) 2021 Dana Basken
--

local AceEvent = LibStub("AceEvent-3.0")

StateMachine = {}
StateMachine.__index = StateMachine

function StateMachine:Create(id, initial)
  local instance = {}
  setmetatable(instance, StateMachine)
  instance:Construct(id, initial)
  return instance
end

function StateMachine:Construct(id, initial)
  self.id = id
  self.initial = initial
  self.when = debugprofilestop()
  self:Reset()
end

function StateMachine:Reset()
  self:SetState(self.initial)
end

function StateMachine:SetState(state)
  if state == nil then state = '' end
  if state ~= self.state then
    local lastState = self.state
    local now = debugprofilestop()
    local lastDurationMs = now - self.when
    self.state = state
    self.when = now
    self:OnStateChange(lastState, lastDurationMs)
  end
end

function StateMachine:DurationMs()
  return debugprofilestop() - self.when
end

function StateMachine:OnStateChange(lastState, lastDurationMs)
  if lastState == nil then lastState = "" end
  AceEvent:SendMessage(("STATE_CHANGE_%s"):format(self.id), self.state, lastState, lastDurationMs)
end
